/* $Id: ulength.h,v 1.5 2006/01/18 12:04:46 cvsroot Exp $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_ULENGTH_INCLUDED
#define GENLIB2_ULENGTH_INCLUDED

#include <string>

using namespace std;

namespace GenLib2
{

/** Unit length class.
 *  Measurements of length.
 *  @author Dirk Stallmann
 */
class UnitLength
{
public:
	/// code of length
	enum ecode
	{
		/// unknown
		UNKNOWN,

		/// kilometer
		KILOMETRE,

		/// metre
		METRE,

		/// centimeter
		CENTIMETRE,

		/// millimetre
		MILLIMETRE,

		/// micrometre
		MICROMETRE,

		/// international mile
		MILE,

		/// US state mile
		MILES_US,

		/// inch
		INCH,

		/// feet
		FEET,

		/// yard
		YARD,

		/// nautical mile
		NAUTICAL_MILE
	};

	/**@name set unit using symbol-string.
	 *
	 *  @param symbol
	 *  @return true = OK, false = error occured
	 */
	bool set(const char* symbol);
	
	/// default constructor
	UnitLength() { set("m"); }

	/**@name constructor using symbol-string.
	 *
	 *  @param symbol
	 */
	UnitLength(const char* symbol) { set(symbol); }
	
	/// destructor
	virtual ~UnitLength() { }
	
	/**@name get scale factor.
	 *
	 *  @return scale factor
	 */
	double scale() const { return _scale; }
	
	/**@name get symbol-string.
	 *
	 *  @return symbol
	 */
	const char* symbol() const { return _map[_code].symbol; }
	
	/**@name get code.
	 *
	 *  @param symbol
	 *  @return code
	 */
	static ecode code(const char* symbol);

	/**@name get name.
	 *
	 *  @param code
	 *  @return name
	 */
	static const char* name(int code);

	/**@name get symbol.
	 *
	 *  @param code
	 *  @return symbol
	 */
	static const char* symbol(int code);

	/**@name get scale factor.
	 *  Get scale factor to convert from the actual unit
	 *  into the SI-base unit meter. The multiplication of the length
	 *  by the scale results to the equivalent length in meter.
	 *
	 *  @param code
	 *  @return scale factor
	 */
	static double scale(int code);

	// conversion from unit base
	//double conv(double v) const { return (v * _scale); }

	// conversion into unit base
	//double ubase(double v) const { return (v / _scale); }

protected:
	/// code
	ecode	_code;
	
	/// scale factor
	double	_scale;
	
	/// map size
	static const int _map_size;

	/// map structure
	struct map_t
	{
		/// code
		int				code;

		/// symbol or short name
		const char*		symbol;

		/// (long) name
		const char*		name;

		/// scale factor
		double			scale;
	};

	/// map
	static map_t _map[];
};


// non-member functions...


// implementation

} // namespace GenLib2

#endif /* GENLIB2_ULENGTH_INCLUDED */
